/*
 * CronManager.java
 *
 * Created on 31. Oktober 2007, 07:50
 *
 */
package de.retrobib.cron;

import java.util.Date;
import java.util.Locale;
import org.apache.log4j.Logger;
import org.ca.utils.misc.DateUtil;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerFactory;
import org.quartz.Trigger;
import org.quartz.TriggerUtils;
import org.quartz.impl.StdSchedulerFactory;

/**
 * Der Cron-Manager verwaltet die zeitlich gesteuerten Jobs, wie den
 * Konsistenz-Checker oder aktualisieren des Lucene-Indexes.
 *
 * @author caschoff
 * @version 1.0
 */
public class CronManager {

    /** Der Manager selbst. */
    public static CronManager cronManager = null;
    /** Der Scheduler. */
    private Scheduler scheduler = null;
    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(CronManager.class);

    /**
     * Creates a new instance of CronManager
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public CronManager()
            throws Exception {
        SchedulerFactory schedulerFactory = new StdSchedulerFactory();

        scheduler = schedulerFactory.getScheduler();

        scheduler.start();
        logger.debug("Scheduler getartet.");

        //
        // Minuten-Jobs...
        //
        {
            JobDetail jobDetail = new JobDetail("minutelyJob", null, MinutelyJobs.class);

            Trigger trigger = TriggerUtils.makeMinutelyTrigger();
            trigger.setStartTime(TriggerUtils.getEvenMinuteDate(new Date()));
            trigger.setName("minuteyTrigger");

            scheduler.scheduleJob(jobDetail, trigger);
            logger.debug("MinutleyJobs gestartet (nächste Ausführung: " + DateUtil.getFormattedDateString(trigger.getNextFireTime().getTime(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT) + ").");
        }

        //
        // Stunden-Jobs...
        //
        {
            JobDetail jobDetail = new JobDetail("hourlyJob", null, HourlyJobs.class);

            Trigger trigger = TriggerUtils.makeHourlyTrigger();
            trigger.setStartTime(TriggerUtils.getEvenHourDate(new Date()));
            trigger.setName("hourlyTrigger");

            scheduler.scheduleJob(jobDetail, trigger);
            logger.debug("MinutleyJobs gestartet (nächste Ausführung: " + DateUtil.getFormattedDateString(trigger.getNextFireTime().getTime(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT) + ").");
        }

        //
        // Tage-Jobs...
        //
        {
            JobDetail jobDetail = new JobDetail("dailyJob", null, DailyJobs.class);

            Trigger trigger = TriggerUtils.makeDailyTrigger(3, 0);
            trigger.setName("dailyTrigger");

            scheduler.scheduleJob(jobDetail, trigger);
            logger.debug("MinutleyJobs gestartet (nächste Ausführung: " + DateUtil.getFormattedDateString(trigger.getNextFireTime().getTime(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT) + ").");
        }

        //
        // Monats-Jobs...
        //
        {
            JobDetail jobDetail = new JobDetail("monthlyJob", null, MonthlyJobs.class);

            Trigger trigger = TriggerUtils.makeMonthlyTrigger(TriggerUtils.LAST_DAY_OF_MONTH, 23, 59);
            trigger.setStartTime(new Date());
            trigger.setName("monthlyTrigger");

            scheduler.scheduleJob(jobDetail, trigger);
            logger.debug("MinutleyJobs gestartet (nächste Ausführung: " + DateUtil.getFormattedDateString(trigger.getNextFireTime().getTime(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT) + ").");
        }
    }

    /**
     * Nur um zu sehen, ob's klappt.
     */
    @Override
    protected void finalize() {
        try {
            logger.debug("Finalisiere Job-Manager....");
            scheduler.shutdown(true);
            logger.debug("... done.");
        } catch (Exception ex) {
            logger.fatal("finalize: " + ex);
        }
    }

    static {
        try {
            logger.debug("Starte Job-Manager...");
            cronManager = new CronManager();
            logger.debug("... done.");
        } catch (Exception ex) {
            logger.fatal("Static: ", ex);
        }
    }

    /*
     * Getter/Setter
     */
    /**
     * Den Scheduler bekommen.
     *
     * @return Der Scheduler.
     */
    public Scheduler getScheduler() {
        return scheduler;
    }
}
