/*
 * AccountLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Accounts.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class AccountLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(AccountLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private AccountLogic() {
    }

    /**
     * Alle Korrektoren bekommen.
     * 
     * @param hiberSession Die Hibernate-Session.
     * 
     * @return Eine Liste mit den Accounts.
     */
    @SuppressWarnings("unchecked")
    public static List<Account> getAllAccounts(Session hiberSession) {
        List<Account> l = (List<Account>) hiberSession.getNamedQuery("Account").list();
        return l;
    }

    /**
     * Alle Korrektoren bekommen, auch gelöschte.
     *
     * @param hiberSession Die Hibernate-Session.
     *
     * @return Eine Liste mit den Accounts.
     */
    @SuppressWarnings("unchecked")
    public static List<Account> getAllAccountsWithDeletions(Session hiberSession) {
        List<Account> l = (List<Account>) hiberSession.getNamedQuery("Account with deletions").list();
        return l;
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Einen Account anlegen.
     *
     * @param account Der Account.
     * @param hiberSession Die Hibernate-Session.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createAccount(Account account, Session hiberSession, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            account.setTimestamp(System.currentTimeMillis());
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + account);
        hiberSession.persist(account);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Einen Account aktualisieren.
     *
     * @param account Der Account.
     * @param hiberSession Die Hibernate-Session.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateAccount(Account account, Session hiberSession, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            account.setTimestamp(System.currentTimeMillis());
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + account);
        hiberSession.update(account);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Einen Account aufgrund der E-Mailadresse bekommen.
     *
     * @param email Die E-Mailadresse.
     * @param hiberSession Die Hibernate-Session.
     * @return Der Account.
     */
    public static Account getAccountByEmail(String email, Session hiberSession) {
        return (Account) hiberSession.getNamedQuery("Account by email").setString("email", email).setMaxResults(1).list().get(0);
    }

    /*
     * Hilfsmethoden für den Import der alten Daten.
     */
    /**
     * Einen Account aufgrund der alten ID bekommen.
     *
     * @param oldId Die alte Id.
     * @param hiberSession Die Hibernate-Session.
     * @return Der Account.
     */
    public static Account getAccountByOldId(String oldId, Session hiberSession) {
        return (Account) hiberSession.getNamedQuery("Account by oldid").setLong("oldid", Long.parseLong(oldId)).setMaxResults(1).list().get(0);
    }
}
