/*
 * AccountsettingLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Accountsetting;
import java.util.Collection;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Accountsettings.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class AccountsettingLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(AccountsettingLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private AccountsettingLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Accountsetting speichern.
     *
     * @param account Der Account.
     * @param hiberSession Die Hibernate-Session.
     * @param name Der Name des Settings.
     * @param value Der Wert des Settings.
     */
    public static void saveAccountsetting(Account account, Session hiberSession, String name, String value) {
        Collection<Accountsetting> col = account.getAccountsettingCollection();
        if ((col != null) && (col.iterator() != null)) {
            for (Accountsetting as : col) {
                if (as.getName().equals(name) && !as.getDeleted()) {
                    as.setValue(value);
                    Transaction tx = hiberSession.beginTransaction();
                    logger.debug("Update: " + as);
                    hiberSession.update(as);
                    tx.commit();

                    return;
                }
            }
        }

        Accountsetting accountSetting = new Accountsetting();

        accountSetting.setAccountid(account);
        accountSetting.setDeleted(false);
        accountSetting.setTimestamp(System.currentTimeMillis());
        accountSetting.setName(name);
        accountSetting.setValue(value);

        Transaction tx = hiberSession.beginTransaction();
        logger.debug("Create " + accountSetting);
        hiberSession.persist(accountSetting);
        tx.commit();
    }

    /**
     * Ein Accountsetting für einen Account bekommen.
     *
     * @param account Der Account.
     * @param name Der Name des Settings.
     * @return Das Setting oder <code>null</code>, wenn das Setting nicht existiert.
     */
    public static Accountsetting getAccountsetting(Account account, String name) {
        for (Accountsetting as : account.getAccountsettingCollection()) {
            if (as.getName().equals(name) && !as.getDeleted()) {
                return as;
            }
        }

        return null;
    }

    /**
     * Ein Accountsetting löschen.
     *
     * @param account Der Account.
     * @param hiberSession Die Hibernate-Session.
     * @param name Der Name des Settings.
     */
    public static void deleteAccountsetting(Account account, Session hiberSession, String name) {
        for (Accountsetting as : account.getAccountsettingCollection()) {
            if (as.getName().equals(name) && !as.getDeleted()) {
                as.setDeleted(true);
                Transaction tx = hiberSession.beginTransaction();
                logger.debug("Update: " + as);
                hiberSession.update(as);
                tx.commit();
            }
        }
    }

    /**
     * Ein Accountsetting aktualisieren.
     *
     * @param accountsetting Das Setting.
     * @param hiberSession Die Hibernate-Session.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateAccountsetting(Accountsetting accountsetting, Session hiberSession, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            accountsetting.setTimestamp(System.currentTimeMillis());
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + accountsetting);
        hiberSession.update(accountsetting);
        if (withTransaction) {
            tx.commit();
        }
    }
}
