/*
 * SeiteLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Teil;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Seiten.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SeiteLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SeiteLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private SeiteLogic() {
    }

    /**
     * Die folgende Seite finden.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param seite Die Seite, zu der die folgende Seite gefunden werden soll.
     * @return Die folgende Seite, oder <code>null</code>, wenn es keine gibt.
     */
    public static Seite next(Session hiberSession, Seite seite) {
        Seite ret = null;
        @SuppressWarnings("unchecked")
        List<Seite> next = (List<Seite>) hiberSession.getNamedQuery("Naechste Seite").setLong("teilid", seite.getTeilid().getId()).setString("sort", seite.getSort()).setMaxResults(1).list();

        if (next.size() > 0) {
            ret = next.get(0);
        } else {
            // eventuell im nächsten Teil?
            @SuppressWarnings("unchecked")
            List<Teil> nextTeilList = (List<Teil>) hiberSession.getNamedQuery("Naechster Teil").setLong("bandid", seite.getTeilid().getBandid().getId()).setString("sort", seite.getTeilid().getSort()).setMaxResults(1).list();
            if (nextTeilList.size() > 0) {
                // ja...
                Teil nextTeil = nextTeilList.get(0);
                @SuppressWarnings("unchecked")
                List<Seite> ersteSeiteList = (List<Seite>) hiberSession.getNamedQuery("Erste Seite").setLong("teilid", nextTeil.getId()).setMaxResults(1).list();
                if (ersteSeiteList.size() > 0) {
                    ret = ersteSeiteList.get(0);
                }
            }
        }

        return ret;
    }

    /**
     * Die vorhergehende Seite finden.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param seite Die Seite, zu der die vorhergehende Seite gefunden werden soll.
     * @return Die vorhergehende Seite, oder <code>null</code>, wenn es keine gibt.
     */
    public static Seite prev(Session hiberSession, Seite seite) {
        Seite ret = null;
        @SuppressWarnings("unchecked")
        List<Seite> prev = (List<Seite>) hiberSession.getNamedQuery("Vorhergehende Seite").setLong("teilid", seite.getTeilid().getId()).setString("sort", seite.getSort()).setMaxResults(1).list();

        if (prev.size() > 0) {
            ret = prev.get(0);
        } else {
            // eventuell im vorhergehenden Teil?
            @SuppressWarnings("unchecked")
            List<Teil> prevTeilList = (List<Teil>) hiberSession.getNamedQuery("Vorhergehender Teil").setLong("bandid", seite.getTeilid().getBandid().getId()).setString("sort", seite.getTeilid().getSort()).setMaxResults(1).list();
            if (prevTeilList.size() > 0) {
                // ja...
                Teil prevTeil = prevTeilList.get(0);
                @SuppressWarnings("unchecked")
                List<Seite> prevSeiteList = (List<Seite>) hiberSession.getNamedQuery("Letzte Seite").setLong("teilid", prevTeil.getId()).setMaxResults(1).list();
                if (prevSeiteList.size() > 0) {
                    ret = prevSeiteList.get(0);
                }
            }
        }

        return ret;
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Eine Seite anlegen.
     *
     * @param seite Die Seite.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createSeite(Seite seite, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            seite.setTimestamp(System.currentTimeMillis());
        }
        seite.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + seite);
        hiberSession.persist(seite);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Eine Seite aktualisieren.
     *
     * @param seite Die Seite.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateSeite(Seite seite, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            seite.setTimestamp(System.currentTimeMillis());
        }

        if (account != null) {
            seite.setAccountid(account);
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + seite);
        hiberSession.update(seite);
        if (withTransaction) {
            tx.commit();
        }
    }
}
