/*
 * TeilLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Teil;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Teile.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class TeilLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(TeilLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private TeilLogic() {
    }

    /**
     * Einen Teil aufgrund eines Bandes und des Namens bekommne.
     *
     * @param band Der Band, zu dem der Teil gehört.
     * @param name Der Name des Teils.
     * @param hiberSession Die Hibernate-Session.
     *
     * @return Der Teil.
     */
    public static Teil getTeilByBandAndName(Band band, String name, Session hiberSession) {
        Teil teil = null;
        @SuppressWarnings("unchecked")
        List<Teil> l = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid and name").setLong("bandid", band.getId()).setString("name", name).list();
        if (l.size() == 1) {
            teil = l.get(0);
        }

        return teil;
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Einen Teil anlegen.
     *
     * @param teil Der Teil.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createTeil(Teil teil, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            teil.setTimestamp(System.currentTimeMillis());
        }
        teil.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + teil);
        hiberSession.persist(teil);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Einen leeren Teil vorbereiten.
     *
     * @param band Der Eltern-Band.
     * @return Der Teil.
     */
    public static Teil createEmptyTeil(Band band) {
        Teil teil = new Teil();

        teil.setBandid(band);
        teil.setDeleted(false);
        teil.setDescription("");
        teil.setName("Temporär");
        teil.setSort("000");
        teil.setVisible(true);

        return teil;
    }

    /**
     * Einen Teil aktualisieren.
     *
     * @param teil Der Teil.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateTeil(Teil teil, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            teil.setTimestamp(System.currentTimeMillis());
        }
        if (account != null) {
            teil.setAccountid(account);
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + teil);
        hiberSession.update(teil);
        if (withTransaction) {
            tx.commit();
        }
    }
}
