/*
 * Manager.java
 *
 * Created on 4. September 2007, 08:08
 *
 */
package de.retrobib.lucene;

import de.retrobib.Constants.LuceneFieldNames;
import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.utils.Mailer;
import de.retrobib.xml.Extractor;
import java.util.List;
import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.spell.LuceneDictionary;
import org.apache.lucene.search.spell.SpellChecker;
import org.apache.lucene.store.FSDirectory;
import org.hibernate.Session;

/**
 * Manager um Dokumente im Lucene-Index zu verwalten.<br />
 * Für den Did you mean...?-Index wird ein ungestemmter temporärer Index geschrieben.
 *
 * 22.11.2007: 1.1 - Auf eigene Analyzer umgestellt, um spätere Wartung zu erleichtern.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class Manager {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Manager.class);
    /** Der IndexWriter für den Hauptindex. */
    private IndexWriter indexWriter = null;
    /** Der IndexWriter für den temporären DidYouMean-Index. */
    private IndexWriter DYMTempIndexWriter = null;
    /** Index neu erstellen? */
    private boolean create = false;

    /**
     * Creates a new instance of Manager.
     *
     * @param create Bei <code>true</code> wird der Index neu angelegt.
     */
    public Manager(boolean create) {
        this.create = create;
    }

    /**
     * Optimieren des Lucene-Indexes.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void optimize()
            throws Exception {
        indexWriter.optimize();
        DYMTempIndexWriter.optimize();
    }

    /**
     * Erstellen des 'Did you mean..._Indexes...
     *
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    public void createDidYouMeanIndex()
            throws Exception {
        logger.debug("Did you mean-Index aufbauen...");
        IndexReader indexReader = IndexReader.open(MiscConstants.luceneDYMTempDir);
        SpellChecker spellChecker = new SpellChecker(FSDirectory.getDirectory(MiscConstants.luceneDYMDir));
        if (create) {
            spellChecker.clearIndex();
        }
        spellChecker.indexDictionary(new LuceneDictionary(indexReader, LuceneFieldNames.didyoumean));
        indexReader.close();
        logger.debug("... beendet.");
    }

    /**
     * Den Writer öffnen.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void open()
            throws Exception {
        if (indexWriter == null) {
            indexWriter = new IndexWriter(MiscConstants.luceneDir, new RetroBibAnalyzer(), create);
            indexWriter.setMaxFieldLength(Integer.MAX_VALUE);
            DYMTempIndexWriter = new IndexWriter(MiscConstants.luceneDYMTempDir, new DidYouMeanAnalyzer(), create);
            DYMTempIndexWriter.setMaxFieldLength(Integer.MAX_VALUE);
        }
    }

    /**
     * Den Writer schliessen.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void close()
            throws Exception {
        if (indexWriter != null) {
            indexWriter.close();
            indexWriter = null;
            DYMTempIndexWriter.close();
            DYMTempIndexWriter = null;
        }
    }

    /**
     * Eine Seite zum Index hinzufügen.
     *
     * @param seite Die Seite.
     * @param hiberSession Die Hibernate-Session.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void addToLucene(Seite seite, Session hiberSession)
            throws Exception {
        try {
            Document document = new Document();
            Document DYMTempDocument = new Document();

            // Felder für das Werk...
            Field werk_name = new Field(LuceneFieldNames.werk_name, seite.getTeilid().getBandid().getWerkid().getName(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO);
            document.add(werk_name);
            Field werk_shortname = new Field(LuceneFieldNames.werk_shortname, seite.getTeilid().getBandid().getWerkid().getShortname(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO);
            document.add(werk_shortname);
            Field werk_id = new Field(LuceneFieldNames.werk_id, "" + seite.getTeilid().getBandid().getWerkid().getId(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(werk_id);
            Field werk_timestamp = new Field(LuceneFieldNames.werk_timestamp, "" + seite.getTeilid().getBandid().getWerkid().getTimestamp(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(werk_timestamp);

            // Felder für den Band...
            Field band_name = new Field(LuceneFieldNames.band_name, seite.getTeilid().getBandid().getName(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO);
            document.add(band_name);
            Field band_id = new Field(LuceneFieldNames.band_id, "" + seite.getTeilid().getBandid().getId(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(band_id);
            Field band_timestamp = new Field(LuceneFieldNames.band_timestamp, "" + seite.getTeilid().getBandid().getTimestamp(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(band_timestamp);

            // Felder für den Teil...
            Field teil_name = new Field(LuceneFieldNames.teil_name, seite.getTeilid().getName(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO);
            document.add(teil_name);
            Field teil_id = new Field(LuceneFieldNames.teil_id, "" + seite.getTeilid().getId(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(teil_id);
            Field teil_timestamp = new Field(LuceneFieldNames.teil_timestamp, "" + seite.getTeilid().getTimestamp(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(teil_timestamp);

            // die Seite
            Field seite_id = new Field(LuceneFieldNames.seite_id, "" + seite.getId(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seite_id);
            Field seite_timestamp = new Field(LuceneFieldNames.seite_timestamp, "" + seite.getTimestamp(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seite_timestamp);
            Field seite_von = new Field(LuceneFieldNames.seite_von, seite.getStartword(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seite_von);
            if (seite.getEndword() != null) {
                Field seite_bis = new Field(LuceneFieldNames.seite_bis, seite.getEndword(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
                document.add(seite_bis);
            } else {
                Field seite_bis = new Field(LuceneFieldNames.seite_bis, "", Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
                document.add(seite_bis);
            }
            Field seite_nummer = new Field(LuceneFieldNames.seite_nummer, seite.getPagenumber(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seite_nummer);
            Field seite_status = new Field(LuceneFieldNames.seite_status, "" + seite.getStatus(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seite_status);
            Field seite_email = new Field(LuceneFieldNames.seite_email, seite.getAccountid().getEmail(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seite_email);

            // der Text
            Seitentext seitentext = (Seitentext) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);
            Field seitentext_id = new Field(LuceneFieldNames.seitentext_id, "" + seitentext.getId(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seitentext_id);
            Field seitentext_timestamp = new Field(LuceneFieldNames.seitentext_timestamp, "" + seitentext.getTimestamp(), Field.Store.YES, Field.Index.UN_TOKENIZED, Field.TermVector.NO);
            document.add(seitentext_timestamp);
            String content = null;

            if (seite.getStatus() != Seite.TAGGED) {
                content = seitentext.getContent();
            } else {
                Extractor extractor = new Extractor(seitentext.getContent());
                content = extractor.getLuceneContent();
            }
            Field seitentext_content = new Field(LuceneFieldNames.seitentext_content, content, Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.YES);
            document.add(seitentext_content);

            // die Schlagworte
            StringBuilder schlagworte_content = new StringBuilder();
            if (seite.getStatus() != Seite.TAGGED) {
                List schlagworte = hiberSession.getNamedQuery("Schlagwort by seiteid order by length").setLong("seiteid", seite.getId()).list();
                for (int i = 0; i < schlagworte.size(); i++) {
                    schlagworte_content.append(((Schlagwort) schlagworte.get(i)).getKeyword());
                    schlagworte_content.append(" ");
                }
                // eventuell noch von/bis hinzufügen
                if ((seite.getStartword() != null) && (schlagworte_content.indexOf(seite.getStartword()) == -1)) {
                    schlagworte_content.append(seite.getStartword());
                    schlagworte_content.append(" ");
                }
                if ((seite.getEndword() != null) && (schlagworte_content.indexOf(seite.getEndword()) == -1)) {
                    schlagworte_content.append(seite.getEndword());
                    schlagworte_content.append(" ");
                }
            } else {
                Extractor extractor = new Extractor(seitentext.getContent());
                schlagworte_content.append(extractor.getBoostWorte());
            }
            Field keywords = new Field(LuceneFieldNames.schlagworte, schlagworte_content.toString(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.YES);
            document.add(keywords);

            // für den Did you mean..?-Index...
            DYMTempDocument.add(seite_id);
            Field didyoumean = new Field(LuceneFieldNames.didyoumean, schlagworte_content.toString(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO);
            DYMTempDocument.add(didyoumean);

            // und in den Index...
            indexWriter.addDocument(document);
            DYMTempIndexWriter.addDocument(DYMTempDocument);
        } catch (Exception ex) {
            logger.error("Probleme (" + ex.getMessage() + ") mit der Seite " + seite, ex);
            new Mailer().send("Nachricht vom Lucene-Index-Builder", "Fehler (" + ex.getMessage() + ") bei der Seite " + seite + " (http://www.retrobibliothek.de/retrobib/seite.html?id=" + seite.getId() + ").", "christian.aschoff@uni-ulm.de", null);
        }

    }

    /**
     * Eine Seite aus dem Index löschen.
     *
     * @param seiten Die Seiten.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void deleteFromLucene(List seiten)
            throws Exception {
        // Achtung! Der IndexWriter muß geschlossen sein!
        IndexReader indexReader = IndexReader.open(MiscConstants.luceneDir);
        IndexReader DYMTempIndexReader = IndexReader.open(MiscConstants.luceneDYMTempDir);

        for (int i = 0; i < seiten.size(); i++) {
            Seite seite = (Seite) seiten.get(i);
            logger.debug("Lösche " + seite + " aus Lucene/DYM...");

            int anzahl = indexReader.deleteDocuments(new Term(LuceneFieldNames.seite_id, "" + seite.getId()));
            logger.debug("... " + anzahl + " Dokumente aus Lucene betroffen.");

            int DYManzahl = DYMTempIndexReader.deleteDocuments(new Term(LuceneFieldNames.seite_id, "" + seite.getId()));
            logger.debug("... " + DYManzahl + " Dokumente aus DYM betroffen.");
        }

        DYMTempIndexReader.close();
        indexReader.close();
    }
}
