/*
 * Sherlock.java
 *
 * Created on 4. September 2007, 11:14
 *
 */
package de.retrobib.lucene;

import de.retrobib.Constants.LuceneFieldNames;
import de.retrobib.Constants.MiscConstants;
import de.retrobib.webelements.LuceneHit;
import de.retrobib.webelements.LuceneHits;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.highlight.Fragmenter;
import org.apache.lucene.search.highlight.Highlighter;
import org.apache.lucene.search.highlight.QueryScorer;
import org.apache.lucene.search.highlight.SimpleFragmenter;
import org.apache.lucene.search.highlight.SimpleHTMLFormatter;
import org.apache.lucene.search.spell.SpellChecker;
import org.apache.lucene.store.FSDirectory;
import org.hibernate.Session;

/**
 * Hilfsklasse zum Suchen im Lucene-Index.
 *
 * 22.11.2007: 1.1 - Aud eigenen Analyzer umgestellt, um spätere Wartung zu erleichtern.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class Sherlock {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Sherlock.class);
    /** Der Reader (statisch und nur einmal, das Öffnen dauert lange!) */
    private static IndexReader indexReader = null;
    /** Der Searcher. */
    private static Searcher searcher = null;
    /** Der Did-You-Mean Index. */
    private static SpellChecker spellChecker = null;
    /** DIe letzten Suchen. */
    private static ArrayList<String> lastQuerys = null;

    /**
     * Alles ist statich...
     */
    private Sherlock() {
    }

    /**
     * Den Reader öffnen.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static void open()
            throws Exception {
        if (searcher == null) {
            logger.debug("Öffne Lucene-IndexReader...");
            indexReader = IndexReader.open(MiscConstants.luceneDir, true);
            searcher = new IndexSearcher(indexReader);
            logger.debug("Öffne Lucene-Did you mean...-Index...");
            spellChecker = new SpellChecker(FSDirectory.getDirectory(MiscConstants.luceneDYMDir));
        }
    }

    /**
     * Den Reader schliessen.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static void close()
            throws Exception {
        logger.debug("Schliesse Lucene-IndexReader...");
        if (searcher != null) {
            searcher.close();
            indexReader.close();

            searcher = null;
            indexReader = null;
            spellChecker = null;
        }
    }

    /**
     * Suchen.
     *
     * @param keywords Die zu suchenden Stichwörter.
     * @param kurznamen Eine Liste mit Kurznamen der Werke, auf die die Suche
     *                  eingeschränkt werden soll.
     * @param hiberSession Die Hibernate-Session.
     * @throws java.lang.Exception Bei einem Fehler.
     * 
     * @return Einen Wrapper um die Lucene-Hits.
     */
    public static synchronized LuceneHits search(String keywords, ArrayList<String> kurznamen, Session hiberSession)
            throws Exception {
        if (lastQuerys == null) {
            lastQuerys = new ArrayList<String>();
        }
        if (!lastQuerys.contains(keywords)) {
            lastQuerys.add(0, keywords);
        }
        if (lastQuerys.size() > 10) {
            lastQuerys.remove(10);
        }

        open();

        String originalKeywords = keywords;

        String addQuery = "";
        if (kurznamen != null) {
            for (int i = 0; i < kurznamen.size(); i++) {
                addQuery += LuceneFieldNames.werk_shortname + ":" + kurznamen.get(i) + " ";
            }
        }
        if (addQuery.length() != 0) {
            addQuery = " +(" + addQuery + ")";
        }

        keywords = "+( (" + LuceneFieldNames.seitentext_content + ":(" + keywords + ")) OR (" + LuceneFieldNames.schlagworte + ":(" + keywords + ")^4) ) " + addQuery;
        logger.debug("Keywords: " + keywords);

        // Standard-Suche...
        RetroBibAnalyzer retrobibAnalyzer = new RetroBibAnalyzer();
        QueryParser queryParser = new QueryParser(LuceneFieldNames.seitentext_content, retrobibAnalyzer);
        queryParser.setDefaultOperator(QueryParser.Operator.AND);
        Query query = queryParser.parse(keywords);
        query = query.rewrite(indexReader);
        logger.debug("Query: " + query.toString());

        long start = System.currentTimeMillis();
        Hits hits = searcher.search(query);
        long end = System.currentTimeMillis();
        logger.debug("Treffer: " + hits.length());

        QueryScorer scorer = new QueryScorer(query);
        SimpleHTMLFormatter simpleHTMLFormatter = new SimpleHTMLFormatter("<em class=\"highlight\">", "</em>");
        Highlighter highlighter = new Highlighter(simpleHTMLFormatter, scorer);
        Fragmenter fragmenter = new SimpleFragmenter(250);
        highlighter.setTextFragmenter(fragmenter);

        LuceneHits luceneHits = new LuceneHits(hits.length(), end - start);
        int max = hits.length();
        if (max > 100) {
            max = 100;
        }
        for (int i = 0; i < max; i++) {
            Document document = hits.doc(i);
            float rank = hits.score(i);
            luceneHits.addHit(new LuceneHit(document, rank, highlighter.getBestFragment(retrobibAnalyzer, LuceneFieldNames.seitentext_content, document.get(LuceneFieldNames.seitentext_content)), hiberSession));
        }

        String[] similars = spellChecker.suggestSimilar(originalKeywords, 1);
        if (((hits.length() == 0) || (hits.score(0) < 0.1)) && (similars.length > 0)) {
            logger.debug("Did you mean '" + similars[0] + "' (you typed '" + originalKeywords + "')?");
            luceneHits.setDidyoumean(similars[0]);
        }

        return luceneHits;
    }

    /**
     * Die letzten Suchbegriffe bekommen.
     *
     * @return Eine ArrayList mit den letzten Suchbegriffen.
     */
    public static ArrayList<String> getLastQuerys() {
        return lastQuerys;
    }
}
