/*
 * UTF8AccentFilter.java
 *
 * Created on 22. November 2007, 12:42
 *
 */
package de.retrobib.lucene;

import java.io.IOException;
import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;

/**
 * Filter für die Entfernung von Zeichen mit Accents
 * und ähnlichen Varianten.<br />
 * So wird aus Đ ein D, aus ŵ ein w etc. Die Groß-/Kleinschreibung
 * wird nicht verändert.<br />
 * Erweiterter Quelltext des ISOLatin1AccentFilter aus dem Lucene
 * Projekt.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class UTF8AccentFilter extends TokenFilter {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(UTF8AccentFilter.class);

    /**
     * Test.
     */
    static {
        String input = "ÀÁÂÃÄÅĀĄĂÇĆČĈĊĎĐÈÉÊËĒĘĚĔĖĜĞĠĢĤĦÌÍÎÏĪĨĬĮİĴĶŁĽĹĻĿÑŃŇŅŊÒÓÔÕÖØŌŐŎŔŘŖŚŠŞŜȘŤŢŦȚÙÚÛÜŪŮŰŬŨŲŴÝŶŸȲŹŽŻÆĲŒ";
        String output = removeAccents(input);
        logger.debug("->: " + input);
        logger.debug("<-: " + output);
        input = "àáâãäåāąăçćčĉċďđèéêëēęěĕėƒĝğġģĥħìíîïīĩĭįıĵķĸłľĺļŀñńňņŉŋòóôõöøōőŏŕřŗśšşŝșťţŧțùúûüūůűŭųŵýÿŷȳžżźæĳœß";
        output = removeAccents(input);
        logger.debug("->: " + input);
        logger.debug("<-: " + output);
    }

    /**
     * Creates a new instance of UTF8AccentFilter.
     *
     * @param input Der TokenStream.
     */
    public UTF8AccentFilter(TokenStream input) {
        super(input);
    }

    /**
     * Get the next token.
     * 
     * @return The next token in the stream, or null at EOS.
     * @throws java.io.IOException Bei einem Fehler.
     */
    public final Token next()
            throws IOException {
        final Token t = input.next();
        if (t != null) {
            String oldText = t.termText();
            String newText = removeAccents(oldText);
            t.setTermText(newText);
        }
        return t;
    }

    /**
     * To replace accented characters in a String by unaccented equivalents.
     * Enhancements to ISOLatin1AccentFilter are marked with *.
     * 
     * @param input The String with characters to replace.
     * @return The String with replaced characters.
     */
    public final static String removeAccents(String input) {
        final StringBuffer output = new StringBuffer();

        for (int i = 0; i < input.length(); i++) {
            switch (input.charAt(i)) {
                // Uppercase letters...
                case '\u00C0': // À
                case '\u00C1': // Á
                case '\u00C2': // Â
                case '\u00C3': // Ã
                case '\u00C4': // Ä
                case '\u00C5': // Å
                case '\u0100': // Ā *
                case '\u0104': // Ą *
                case '\u0102': // Ă *
                    output.append('A');
                    break;
                case '\u00C6': // Æ
                    output.append("AE");
                    break;
                case '\u00C7': // Ç
                case '\u0106': // Ć *
                case '\u010C': // Č *
                case '\u0108': // Ĉ *
                case '\u010A': // Ċ *
                    output.append('C');
                    break;
                case '\u00D0': // Ð
                case '\u010E': // Ď *
                case '\u0110': // Đ *
                    output.append('D');
                    break;
                case '\u00C8': // È
                case '\u00C9': // É
                case '\u00CA': // Ê
                case '\u00CB': // Ë
                case '\u0112': // Ē *
                case '\u0118': // Ę *
                case '\u011A': // Ě *
                case '\u0114': // Ĕ *
                case '\u0116': // Ė *
                    output.append('E');
                    break;
                case '\u011C': // Ĝ *
                case '\u011E': // Ğ *
                case '\u0120': // Ġ *
                case '\u0122': // Ģ *
                    output.append('G');
                    break;
                case '\u0124': // Ĥ *
                case '\u0126': // Ħ *
                    output.append('H');
                    break;
                case '\u00CC': // Ì
                case '\u00CD': // Í
                case '\u00CE': // Î
                case '\u00CF': // Ï
                case '\u012A': // Ī *
                case '\u0128': // Ĩ *
                case '\u012C': // Ĭ *
                case '\u012E': // Į *
                case '\u0130': // İ *
                    output.append('I');
                    break;
                case '\u0132': // Ĳ *
                    output.append("IJ");
                    break;
                case '\u0134': // Ĵ *
                    output.append('J');
                    break;
                case '\u0136': // Ķ *
                    output.append('K');
                    break;
                case '\u0141': // Ł *
                case '\u013D': // Ľ *
                case '\u0139': // Ĺ *
                case '\u013B': // Ļ *
                case '\u013F': // Ŀ *
                    output.append('L');
                    break;
                case '\u00D1': // Ñ
                case '\u0143': // Ń *
                case '\u0147': // Ň *
                case '\u0145': // Ņ *
                case '\u014A': // Ŋ *
                    output.append('N');
                    break;
                case '\u00D2': // Ò
                case '\u00D3': // Ó
                case '\u00D4': // Ô
                case '\u00D5': // Õ
                case '\u00D6': // Ö
                case '\u00D8': // Ø
                case '\u014C': // Ō *
                case '\u0150': // Ő *
                case '\u014E': // Ŏ *
                    output.append('O');
                    break;
                case '\u0152': // Œ
                    output.append("OE");
                    break;
                case '\u0154': // Ŕ *
                case '\u0158': // Ř *
                case '\u0156': // Ŗ *
                    output.append('R');
                    break;
                case '\u015A': // Ś *
                case '\u0160': // Š *
                case '\u015E': // Ş *
                case '\u015C': // Ŝ *
                case '\u0218': // Ș *
                    output.append('S');
                    break;
                case '\u0164': // Ť *
                case '\u0162': // Ţ *
                case '\u0166': // Ŧ *
                case '\u021A': // Ț *
                    output.append('T');
                    break;
                case '\u00DE': // Þ
                    output.append("TH");
                    break;
                case '\u00D9': // Ù
                case '\u00DA': // Ú
                case '\u00DB': // Û
                case '\u00DC': // Ü
                case '\u016A': // Ū *
                case '\u016E': // Ů *
                case '\u0170': // Ű *
                case '\u016C': // Ŭ *
                case '\u0168': // Ũ *
                case '\u0172': // Ų *
                    output.append('U');
                    break;
                case '\u0174': // Ŵ *
                    output.append('W');
                    break;
                case '\u00DD': // Ý
                case '\u0176': // Ŷ *
                case '\u0178': // Ÿ
                case '\u0232': // Ȳ *
                    output.append('Y');
                    break;
                case '\u0179': // Ź *
                case '\u017D': // Ž *
                case '\u017B': // Ż *
                    output.append('Z');
                    break;

                // Lowercase letters...
                case '\u00E0': // à
                case '\u00E1': // á
                case '\u00E2': // â
                case '\u00E3': // ã
                case '\u00E4': // ä
                case '\u00E5': // å
                case '\u0101': // ā *
                case '\u0105': // ą *
                case '\u0103': // ă *
                    output.append('a');
                    break;
                case '\u00E6': // æ
                    output.append("ae");
                    break;
                case '\u00E7': // ç
                case '\u0107': // ć *
                case '\u010D': // č *
                case '\u0109': // ĉ *
                case '\u010B': // ċ *
                    output.append('c');
                    break;
                case '\u00F0': // ð
                case '\u010F': // ď *
                case '\u0111': // đ *
                    output.append('d');
                    break;
                case '\u00E8': // è
                case '\u00E9': // é
                case '\u00EA': // ê
                case '\u00EB': // ë
                case '\u0113': // ē *
                case '\u0119': // ę *
                case '\u011B': // ě *
                case '\u0115': // ĕ *
                case '\u0117': // ė *
                    output.append('e');
                    break;
                case '\u0192': // ƒ *
                    output.append('f');
                    break;
                case '\u011D': // ĝ *
                case '\u011F': // ğ *
                case '\u0121': // ġ *
                case '\u0123': // ģ *
                    output.append('g');
                    break;
                case '\u0125': // ĥ *
                case '\u0127': // ħ *
                    output.append('h');
                    break;
                case '\u00EC': // ì
                case '\u00ED': // í
                case '\u00EE': // î
                case '\u00EF': // ï
                case '\u012B': // ī *
                case '\u0129': // ĩ *
                case '\u012D': // ĭ *
                case '\u012F': // į *
                case '\u0131': // ı *
                    output.append('i');
                    break;
                case '\u0133': // ĳ *
                    output.append("ij");
                    break;
                case '\u0135': // ĵ *
                    output.append('j');
                    break;
                case '\u0137': // ķ *
                case '\u0138': // ĸ *
                    output.append('k');
                    break;
                case '\u0142': // ł *
                case '\u013E': // ľ *
                case '\u013A': // ĺ *
                case '\u013C': // ļ *
                case '\u0140': // ŀ *
                    output.append('l');
                    break;
                case '\u00F1': // ñ
                case '\u0144': // ń *
                case '\u0148': // ň *
                case '\u0146': // ņ *
                case '\u0149': // ŉ *
                case '\u014B': // ŋ *
                    output.append('n');
                    break;
                case '\u00F2': // ò
                case '\u00F3': // ó
                case '\u00F4': // ô
                case '\u00F5': // õ
                case '\u00F6': // ö
                case '\u00F8': // ø
                case '\u014D': // ō *
                case '\u0151': // ő *
                case '\u014F': // ŏ *
                    output.append('o');
                    break;
                case '\u0153': // œ
                    output.append("oe");
                    break;
                case '\u0155': // ŕ *
                case '\u0159': // ř *
                case '\u0157': // ŗ *
                    output.append('r');
                    break;
                case '\u015B': // ś *
                case '\u0161': // š *
                case '\u015F': // ş *
                case '\u015D': // ŝ *
                case '\u0219': // ș *
                    output.append('s');
                    break;
                case '\u00DF': // ß
                    output.append("ss");
                    break;
                case '\u0165': // ť *
                case '\u0163': // ţ *
                case '\u0167': // ŧ *
                case '\u021B': // ț *
                    output.append('t');
                    break;
                case '\u00FE': // þ
                    output.append("th");
                    break;
                case '\u00F9': // ù
                case '\u00FA': // ú
                case '\u00FB': // û
                case '\u00FC': // ü
                case '\u016B': // ū *
                case '\u016F': // ů *
                case '\u0171': // ű *
                case '\u016D': // ŭ *
                case '\u0169': // ũ *
                case '\u0173': // ų *
                    output.append('u');
                    break;
                case '\u0175': // ŵ *
                    output.append('w');
                    break;
                case '\u00FD': // ý
                case '\u00FF': // ÿ
                case '\u0177': // ŷ *
                case '\u0233': // ȳ *
                    output.append('y');
                    break;
                case '\u017E': // ž *
                case '\u017C': // ż *
                case '\u017A': // ź *
                    output.append('z');
                    break;
                default:
                    output.append(input.charAt(i));
                    break;
            }
        }

        return output.toString();
    }
}
