/*
 * Converter.java
 *
 * Created on 15. November 2007, 13:31
 *
 */
package de.retrobib.xml;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.db.model.Siehewort;
import java.util.List;
import org.apache.log4j.Logger;
import org.ca.utils.misc.TextUtil;
import org.hibernate.Session;

/**
 * Der Converter wandelt Textseiten in XML nach der Meyers-DTD um. Da die Informationen 
 * in der Datenbank nicht eindeutig sind, ist das Ergebnis zwar DTD-konform, aber nicht 
 * immer perfekt.
 *
 * 19.11.2007: Content nicht mehr XML-Encoden, stört beim Bearbeiten.
 *
 * @author caschoff
 * @version 1.1
 */
public class Converter {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Converter.class);

    /** Alles ist statisch, kein Konstruktor nötig. */
    private Converter() {
    }

    /**
     * Eine Seite in XML konvertieren.
     * 
     * @return Einen <code>String</code> mit dem XML-Dokuemnt.
     * @param hiberSession Die Hibernate-Session.
     * @param seite Die zu konvertierende Seite.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static String convert(Session hiberSession, Seite seite)
            throws Exception {
        StringBuilder ret = new StringBuilder();

        Seitentext seitentext = (Seitentext) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);

        //
        // trailer...
        //
        ret.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        ret.append("\n");
        ret.append("<!DOCTYPE werk SYSTEM \"" + MiscConstants.baseURL + "/retrobib/xml/meyers.dtd\">\n");
        ret.append("\n");
        ret.append("<werk kurzname=\"" + seite.getTeilid().getBandid().getWerkid().getShortname() + "\">\n");
        ret.append("    <band nummer=\"" + seite.getTeilid().getBandid().getNumber() + "\">\n");
        ret.append("        <teil name=\"" + seite.getTeilid().getName() + "\">\n");
        ret.append("            <seite nummer=\"" + seite.getPagenumber() + "\" von=\"" + seite.getStartword() + "\"");
        if (seite.getEndword() != null) {
            ret.append(" bis=\"" + seite.getEndword() + "\"");
        }
        ret.append(">\n");

        //
        // eigentlich interessantes...
        //
        String text = TextUtil.transcode(seitentext.getContent(), seitentext.getEncoding(), "UTF-8");
        @SuppressWarnings("unchecked")
        List<Schlagwort> schlagworte = hiberSession.getNamedQuery("Schlagwort by seiteid order by length").setLong("seiteid", seite.getId()).list();
        @SuppressWarnings("unchecked")
        List<Siehewort> sieheworte = hiberSession.getNamedQuery("Siehewort by seiteid order by length").setLong("seiteid", seite.getId()).list();

        text = text.replaceAll("\r\n", "\n");
        String[] tokens = text.split("\n");

        if (tokens.length > 0) {
            // Seitennummer
            ret.append("                <seitenzahl>\n");
            ret.append("                    " + tokens[0] + "\n");
            ret.append("                </seitenzahl>\n\n");
        }

        if (tokens.length > 2) {
            // von-bis
            ret.append("                <seitenkopf>\n");
            ret.append("                    " + tokens[2] + "\n");
            ret.append("                </seitenkopf>\n\n");
        }

        for (int t = 4; t < tokens.length; t++) {
            String paraText = tokens[t];
            if (paraText.trim().length() != 0) {
                boolean found = false;
                // Keyword am Anfang?
                for (Schlagwort sw : schlagworte) {
                    String schlagwort = sw.getKeyword();
                    if (paraText.startsWith(schlagwort)) {
                        if ((paraText.length() == schlagwort.length()) // ParaText == Schlagwort
                                || (!Character.isLetter(paraText.charAt(schlagwort.length()))) // Schlagwort nur, wenn kein Buchstabe am Ende+1
                                ) {
                            String rest = paraText.substring(schlagwort.length(), paraText.length());
                            rest = tagSieheWorte(rest, sieheworte);
                            paraText = "<schlagwort>" + TextUtil.encodeXML(schlagwort) + "</schlagwort>" + rest;
                            found = true;
                            break;
                        }
                    }
                }
                // ohne Keyworte am Anfang
                if (!found) {
                    ret.append("                <artikelteil>\n");
                    paraText = tagSieheWorte(paraText, sieheworte);
                    ret.append("                    " + paraText + "\n");
                    ret.append("                </artikelteil>\n");
                } else {
                    // mit Keyorten am Anfang
                    ret.append("                <artikel>\n");
                    ret.append("                    " + paraText + "\n");
                    ret.append("                </artikel>\n\n");
                }
            }
        }

        //
        // footer...
        //
        ret.append("            </seite>\n");
        ret.append("        </teil>\n");
        ret.append("    </band>\n");
        ret.append("</werk>\n");

        return ret.toString();
    }

    /**
     * Rekursive Funktion zum taggen der Verweise in einem Text.
     *
     * @return Der getaggte Text.
     * @param text Der zu taggende Text.
     * @param sieheworte Die Liste der Sieheworte.
     *
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    public static String tagSieheWorte(String text, List<Siehewort> sieheworte)
            throws Exception {
        StringBuilder ret = new StringBuilder();
        boolean found = false;

        for (Siehewort sw : sieheworte) {
            String siehewort = sw.getSeeword();
            int pos = text.indexOf(siehewort);
            if (pos != -1) {
                // Treffer, kann aber in einem Teilwort aufgetaucht sein
                if (pos == text.length() - siehewort.length()) {
                    // ganz am Ende
                    ret.append(tagSieheWorte(text.substring(0, pos), sieheworte));
                    ret.append("<verweis>" + TextUtil.encodeXML(siehewort) + "</verweis>");
                    found = true;
                    break;
                } else if (pos == 0) {
                    // ganz am Anfang
                    ret.append("<verweis>" + TextUtil.encodeXML(siehewort) + "</verweis>");
                    ret.append(tagSieheWorte(text.substring(siehewort.length(), text.length()), sieheworte));
                    found = true;
                    break;
                } else {
                    if (Character.isLetter(text.charAt(pos + siehewort.length()))) {
                        // in Wort gefunden, Fehltreffer
                    } else if (Character.isLetter(text.charAt(pos - 1))) {
                        // in Wort gefunden, Fehltreffer
                    } else {
                        // korrekter Treffer
                        ret.append(tagSieheWorte(text.substring(0, pos), sieheworte));
                        ret.append("<verweis>" + TextUtil.encodeXML(siehewort) + "</verweis>");
                        ret.append(tagSieheWorte(text.substring(pos + siehewort.length(), text.length()), sieheworte));
                        found = true;
                        break;
                    }
                }
            }
        }
        if (!found) {
            ret.append(text);
        }

        return ret.toString();
    }
}
