/*
 * ExtractorContentHandler.java
 *
 * Created on 12. November 2007, 11:31
 *
 */
package de.retrobib.xml;

import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Der ExtractorContentHandler bekommt die Rückmeldungen vom Parser
 * über das Auftreten von Elementen.
 *
 * @author caschoff
 * @version 1.0
 */
public class ExtractorContentHandler implements ContentHandler {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(ExtractorContentHandler.class);
    /** Der Text für Lucene. */
    private StringBuffer luceneContent = new StringBuffer();
    /** Die Schlagwörter. */
    private ArrayList<String> schlagWorte = new ArrayList<String>();
    /** Die boost-Wörter für Lucene (und den Did-You-Mean-Index...). */
    private ArrayList<String> boostWorte = new ArrayList<String>();
    /** Leerzeichen zwischen Character-Daten einfügen? */
    private boolean spaces = true;
    /** Schlagwort-Sammler. */
    private StringBuffer schlagWort = null;
    /** Boost-Wort-Sammler. */
    private StringBuffer boostWort = null;

    /**
     * Creates a new instance of ExtractorContentHandler
     */
    public ExtractorContentHandler() {
    }

    /**
     * Den Inhalt des XML-Dokuemnts für Lucene bekommen.
     *
     * @return Der Inhalt des XML-Dokuemnts für Lucene.
     */
    public String getLuceneContent() {
        return luceneContent.toString();
    }

    /**
     * Die BoostWorte (für den Lucene-Index) aus dem XML-Dokument bekommen.
     *
     * @return Die BoostWorte.
     */
    public String getBoostWorte() {
        StringBuilder ret = new StringBuilder();

        for (String bw : boostWorte) {
            ret.append(bw);
            ret.append(" ");
        }

        return ret.toString();
    }

    /**
     * Eine Liste mit den Schlagwörtern des XML-Dokuments bekommen.
     *
     * @return Eine <code>ArrayList</code> mit den Schlagworten.
     */
    public ArrayList<String> getSchlagworte() {
        return schlagWorte;
    }

    /**
     * Callback um den Document-Locator zu setzten.
     *
     * @param locator Der Locator.
     */
    public void setDocumentLocator(Locator locator) {
    }

    /**
     * Callback beim Start eines Dokuments.
     *
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startDocument()
            throws SAXException {
    }

    /**
     * Callback beim Ende des Dokuments.
     *
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endDocument()
            throws SAXException {
    }

    /**
     * Callback beim Start eines Prefix-Mappings.
     *
     * @param prefix Der Prefix.
     * @param uri Die URI.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
    }

    /**
     * Callback beim Ende eines Prefix-Mappings.
     *
     * @param prefix Der Prefix.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endPrefixMapping(String prefix)
            throws SAXException {
    }

    /**
     * Callback beim Start eines Elements.
     *
     * @param namespaceURI Die Namespace-URI.
     * @param localName Der lokale Name.
     * @param qName Der qualifizierte Name.
     * @param atts Die Attribute.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    @SuppressWarnings("fallthrough")
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
            throws SAXException {
        switch (XmlNames.element(localName)) {
            // START in chemischen Formeln keine Leerzeichen
            case XmlNames.en_chemischeformel:
                spaces = false;
                break;
            case XmlNames.en_schlagwort:
                if (schlagWort == null) {
                    schlagWort = new StringBuffer();
                }
                // Ein Schlagwort kann das Wort im Attribut haben
                if (atts.getValue("", "wort") != null) {
                    schlagWorte.add(atts.getValue("", "wort"));
                    // Chars nicht aufsammeln bis zum Ende des Tags...
                    schlagWort = null;
                }
            case XmlNames.en_artikelteilueberschrift:
            case XmlNames.en_unterschlagwort:
            case XmlNames.en_hinweis:
                if (boostWort == null) {
                    boostWort = new StringBuffer();
                }
                break;
        }
    }

    /**
     * Callback beim Ende eines Elements.
     *
     * @param namespaceURI Die Namespace-URI.
     * @param localName Der lokale Name.
     * @param qName Der qualifizierte Name.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    @SuppressWarnings("fallthrough")
    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
        switch (XmlNames.element(localName)) {
            // ENDE in chemischen Formeln keine Leerzeichen
            case XmlNames.en_chemischeformel:
                spaces = true;
                luceneContent.append(" ");
                break;
            case XmlNames.en_schlagwort:
                if (schlagWort != null) {
                    logger.debug("schlagWorte.add(" + schlagWort.toString() + ");");
                    schlagWorte.add(schlagWort.toString());
                    schlagWort = null;
                }
            case XmlNames.en_artikelteilueberschrift:
            case XmlNames.en_unterschlagwort:
            case XmlNames.en_hinweis:
                if (boostWort != null) {
                    logger.debug("boostWorte.add(" + boostWort.toString() + ");");
                    boostWorte.add(boostWort.toString());
                    boostWort = null;
                }
                break;
        }
    }

    /**
     * Callback wenn Character-Daten auftreten.
     *
     * @param ch Die Characters als Array.
     * @param start Der Startpunkt der Zeichenfolge <i>innerhalb</i> des Arrays.
     * @param length Die Länge der Zeichenfolge <i>ab dem Startpunkt</i>.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        String content = new String(ch, start, length); //.trim();

        if (content.length() != 0) {
            // zum Content für Lucene hinzu...
            luceneContent.append(content);
            if (spaces) {
                luceneContent.append(" ");
            }

            // zu den Boost-Worten hinzu?
            if (boostWort != null) {
                boostWort.append(content);
            }

            // zu den Schlag-Worte hinzufügen?
            if (schlagWort != null) {
                schlagWort.append(content);
            }
        }
    }

    /**
     * Callback wenn ignorierbare Whitespaces auftreten.
     *
     * @param ch Die Characters als Array.
     * @param start Der Startpunkt der Zeichenfolge <i>innerhalb</i> des Arrays.
     * @param length Die Länge der Zeichenfolge <i>ab dem Startpunkt</i>.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void ignorableWhitespace(char[] ch, int start, int length)
            throws SAXException {
    }

    /**
     * Callback bei einer Processing-Instruction.
     *
     * @param target Das Ziel.
     * @param data Die Daten.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void processingInstruction(String target, String data)
            throws SAXException {
    }

    /**
     * Callback wenn eine Entität übersprungen wird.
     *
     * @param name Der Name der Entität.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void skippedEntity(String name)
            throws SAXException {
    }
}
